import { IPersistenceMethod, IProduce } from "../common";
export declare type PersistenceStoragePlugin<T = any> = {
    getState: () => void;
    setState: (state: T) => void;
    current: Pick<ConnectPersistenceStorageArgs, "persistenceKey" | "isSSR" | "persistenceMethod" | "debounceWait"> & {
        isConnected: boolean;
    };
};
export declare type IPersistenceOptions = {
    /**
     * The amount of milliseconds for the
     * the debouncing of the calls to the
     * persistence method, by default 3000 ms
     *
     * @type {number}
     */
    debounceWait?: number;
    /**
     * The key used in the persistence method.
     * If not specified, it uses the "devName"
     *
     * @type {string}
     */
    persistenceKey?: string;
    /**
     * Flag to activate or deactivate the
     * persistence.
     * "false" by default.
     *
     * @type {boolean}
     */
    isActive?: boolean;
    /**
     * Persistence method used, by default it
     * uses localStorage
     *
     * @type {IPersistenceMethod}
     */
    persistenceMethod?: IPersistenceMethod;
    /**
     * Flag used to specify that this store
     * is going to be used in server side
     * rendering context, this flag prevents
     * client/server mismatched html
     * on client side hydration
     *
     * "false" by default.
     * @type {boolean}
     */
    isSSR?: boolean;
};
export declare type ConnectPersistenceStorageArgs = {
    persistenceKey: string;
    produce: IProduce<any>;
    debounceWait?: number;
    persistenceMethod?: IPersistenceMethod | undefined;
    isSSR?: boolean;
};
export declare const connectPersistenceStorage: ({ persistenceKey: persistenceName, produce, debounceWait, persistenceMethod, isSSR, }: ConnectPersistenceStorageArgs) => PersistenceStoragePlugin<any> | null;
