import { createSelector } from 'reselect';
export { createSelector } from 'reselect';
import { enablePatches, produceWithPatches, produce, createDraft, finishDraft, applyPatches } from 'immer';
export { castDraft, castImmutable, enableAllPlugins, enableES5, enableMapSet, original } from 'immer';
import { useCallback, useLayoutEffect, useEffect, useState, useMemo, useRef, createContext, memo, createElement, useContext } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/* eslint react-hooks/exhaustive-deps: 0 */
enablePatches();
var useIsomorphicLayoutEffect = typeof window !== "undefined" ? useLayoutEffect : useEffect;
function toAnonFunction(arg) {
  if (typeof arg === "function") return arg;
  return function () {
    return arg;
  };
}

var incrementParameter = function incrementParameter(num) {
  return ++num;
};

var emptyArray = [];
var useUpdate = function useUpdate(persistencePlugin) {
  var _useState = useState(0),
      setState = _useState[1];

  useEffect(function () {
    if (persistencePlugin === null || persistencePlugin === void 0 ? void 0 : persistencePlugin.current.isSSR) {
      persistencePlugin.getState();
    }
  }, emptyArray);
  return useCallback(function () {
    return setState(incrementParameter);
  }, emptyArray);
};
var isClientSide = typeof window !== "undefined";

var connectDevTools = function connectDevTools(name) {
  try {
    if (typeof window !== "undefined" && typeof window.__REDUX_DEVTOOLS_EXTENSION__ !== "undefined") {
      var devTools = window.__REDUX_DEVTOOLS_EXTENSION__.connect({
        name: "Store " + name,
        serialize: true
      });

      return devTools;
    }
  } catch (err) {}

  return undefined;
};

var connectPersistenceStorage = function connectPersistenceStorage(_ref) {
  var persistenceName = _ref.persistenceKey,
      produce = _ref.produce,
      _ref$debounceWait = _ref.debounceWait,
      debounceWait = _ref$debounceWait === void 0 ? 3000 : _ref$debounceWait,
      _ref$persistenceMetho = _ref.persistenceMethod,
      persistenceMethod = _ref$persistenceMetho === void 0 ? isClientSide ? window.localStorage : undefined : _ref$persistenceMetho,
      isSSR = _ref.isSSR;
  if (!persistenceMethod) return null;
  var isConnected = false;
  var timeout;

  var setStateFn = function setStateFn(state) {
    persistenceMethod.setItem(persistenceName, JSON.stringify(state));
    if (!isConnected) isConnected = true;
  };

  var setState = function setState(state) {
    var execLater = function execLater() {
      timeout = undefined;
      setStateFn(state);
    };

    if (timeout !== undefined) clearTimeout(timeout);
    timeout = setTimeout(execLater, debounceWait);
  };

  var getState = function getState() {
    if (isConnected || !persistenceMethod) return;

    try {
      var state = persistenceMethod.getItem(persistenceName);

      if (state == null) {
        isConnected = true;
        return;
      }

      produce(function () {
        return _extends({}, produce(), {}, JSON.parse(state));
      });
      isConnected = true;
    } catch (err) {}
  };

  if (!isSSR) getState();
  return {
    getState: getState,
    setState: setState,
    current: {
      persistenceKey: persistenceName,
      persistenceMethod: persistenceMethod,
      isSSR: isSSR,
      isConnected: isConnected,
      debounceWait: debounceWait
    }
  };
};

/**
 *  Create **react-state-selector** global store
 *
 * @export
 * @template TStore
 * Any object, including array or complex objects like Map, Set or Classes. Even primitives would work.
 * @template THooks
 * Object that defines the custom hooks of the store.
 * @template TActions
 * Object that defines the custom actions of the store.
 * @template TAsyncActions
 * Object that defines the custom async actions of the store.
 * @param {Immutable<TStore>} initialStore
 * Initial state of the store
 * @param {{
 *     hooks?: THooks;
 *     actions?: TActions;
 *     devName?: string;
 *     devToolsInProduction?: boolean;
 *   }} [options]
 * Options object.
 *
 * @returns {{
 *   useStore: IUseStore<TStore>;
 *   produce: IProduce<TStore>;
 *   asyncProduce: IAsyncProduce<TStore>;
 *   hooks: IHooksObj<TStore, typeof options>;
 *   actions: IActionsObj<TStore, typeof options> & IAsyncActionsObj<TStore, typeof options>;
 * }}
 * Resulting object of the store.
 */

function createStore(initialStore, options) {
  var _options$storagePersi;

  if (process.env.NODE_ENV !== "production") {
    for (var name in options === null || options === void 0 ? void 0 : options.hooks) {
      if (name.length < 4 || name.slice(0, 3) !== "use" || name.charAt(3) === name.charAt(3).toLowerCase()) {
        throw new Error("All hooks should follow the rules of hooks for naming and \"" + name + "\" doesn't");
      }
    }
  }

  if (process.env.NODE_ENV !== "production") {
    if ((options === null || options === void 0 ? void 0 : options.actions) && options.asyncActions) {
      var asyncActionsKeys = Object.keys(options.asyncActions);

      for (var key in options.actions) {
        if (asyncActionsKeys.includes(key)) {
          throw new Error("All the actions and asyncActions should have different names and \"" + key + "\" exists in both objects!");
        }
      }
    }
  }

  var devTools;
  var localStoragePlugin;

  if ((options === null || options === void 0 ? void 0 : options.devName) && (options.devToolsInProduction || process.env.NODE_ENV !== "production")) {
    devTools = connectDevTools(options.devName);
  }

  var listeners = new Map();
  var currentStore = initialStore;

  if (devTools) {
    devTools.init(currentStore);
  }

  var useStore = function useStore() {
    var update = useUpdate(localStoragePlugin);
    useIsomorphicLayoutEffect(function () {
      var globalListener = createSelector(function (s) {
        return s;
      }, function () {
        update();
      });
      listeners.set(globalListener, null);
      return function () {
        listeners["delete"](globalListener);
      };
    }, emptyArray);
    return currentStore;
  };

  var produceObj = {
    produce: function produce$1(draft) {
      var _localStoragePlugin;

      if (typeof draft !== "function") return currentStore;

      if (devTools) {
        var produceFn = produceWithPatches(draft);
        var produceResult = produceFn(currentStore);
        currentStore = produceResult[0];
        devTools.send({
          type: "produce",
          payload: produceResult[1]
        }, currentStore);
      } else {
        var _produceFn = produce(draft);

        currentStore = _produceFn(currentStore);
      }

      listeners.forEach(function (props, listener) {
        listener(currentStore, props);
      });
      (_localStoragePlugin = localStoragePlugin) === null || _localStoragePlugin === void 0 ? void 0 : _localStoragePlugin.setState(currentStore);
      return currentStore;
    },
    asyncProduce: function (draft) {
      try {
        if (typeof draft !== "function") return Promise.resolve(currentStore);
        var storeDraft = createDraft(currentStore);
        return Promise.resolve(Promise.resolve(draft(storeDraft))).then(function () {
          var _localStoragePlugin2;

          finishDraft(storeDraft, function (changes) {
            if (changes.length) {
              currentStore = applyPatches(currentStore, changes);

              if (devTools) {
                devTools.send({
                  type: "asyncProduce",
                  payload: changes
                }, currentStore);
              }

              listeners.forEach(function (props, listener) {
                listener(currentStore, props);
              });
            }
          });
          (_localStoragePlugin2 = localStoragePlugin) === null || _localStoragePlugin2 === void 0 ? void 0 : _localStoragePlugin2.setState(currentStore);
          return currentStore;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }
  };

  if (options === null || options === void 0 ? void 0 : (_options$storagePersi = options.storagePersistence) === null || _options$storagePersi === void 0 ? void 0 : _options$storagePersi.isActive) {
    if (typeof initialStore !== "object" || Array.isArray(initialStore)) throw new Error("For local storage persistence your store has to be an object");
    var persistenceKey = options.storagePersistence.persistenceKey || options.devName;
    if (!persistenceKey) throw new Error("You have to specify persistence key or devName");
    localStoragePlugin = connectPersistenceStorage({
      persistenceKey: persistenceKey,
      produce: produceObj.produce,
      debounceWait: options.storagePersistence.debounceWait,
      persistenceMethod: options.storagePersistence.persistenceMethod,
      isSSR: options.storagePersistence.isSSR
    });
  }

  var actionsObj = {};
  var asyncActionsObj = {};

  var _loop = function _loop() {
    var _Object$entries$_i = _Object$entries[_i],
        actionName = _Object$entries$_i[0],
        actionFn = _Object$entries$_i[1];

    actionsObj[actionName] = function () {
      var _localStoragePlugin3;

      var actionDraft = actionFn.apply(void 0, arguments);

      if (devTools) {
        var produceFn = produceWithPatches(actionDraft);
        var produceResult = produceFn(currentStore);
        currentStore = produceResult[0];
        devTools.send({
          type: actionName,
          payload: produceResult[1]
        }, currentStore);
      } else {
        var _produceFn2 = produce(actionDraft);

        currentStore = _produceFn2(currentStore);
      }

      listeners.forEach(function (props, listener) {
        listener(currentStore, props);
      });
      (_localStoragePlugin3 = localStoragePlugin) === null || _localStoragePlugin3 === void 0 ? void 0 : _localStoragePlugin3.setState(currentStore);
      return currentStore;
    };
  };

  for (var _i = 0, _Object$entries = Object.entries((options === null || options === void 0 ? void 0 : options.actions) || {}); _i < _Object$entries.length; _i++) {
    _loop();
  }

  var _loop2 = function _loop2() {
    var _Object$entries2$_i = _Object$entries2[_i2],
        actionName = _Object$entries2$_i[0],
        actionFn = _Object$entries2$_i[1];

    asyncActionsObj[actionName] = function () {
      try {
        return Promise.resolve(actionFn(produceObj.produce).apply(void 0, arguments)).then(function () {
          return currentStore;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };
  };

  for (var _i2 = 0, _Object$entries2 = Object.entries((options === null || options === void 0 ? void 0 : options.asyncActions) || {}); _i2 < _Object$entries2.length; _i2++) {
    _loop2();
  }

  var hooksObj = {};

  var _loop3 = function _loop3() {
    var _Object$entries3$_i = _Object$entries3[_i3],
        hookName = _Object$entries3$_i[0],
        hookSelector = _Object$entries3$_i[1];

    hooksObj[hookName] = function (hooksProps, hookPropsDeps) {
      var update = useUpdate(localStoragePlugin);
      var props = useMemo(toAnonFunction(hooksProps), hookPropsDeps || [hooksProps]);
      var isMountedRef = useRef(false);

      var _useMemo = useMemo(function () {
        return {
          updateSelector: createSelector(hookSelector, function (result) {
            stateRef.current = result;

            if (!isMountedRef.current) {
              return;
            }

            update();
          }),
          initialStateRef: hookSelector(currentStore, props)
        };
      }, emptyArray),
          updateSelector = _useMemo.updateSelector,
          initialStateRef = _useMemo.initialStateRef;

      var stateRef = useRef(initialStateRef);
      useIsomorphicLayoutEffect(function () {
        updateSelector(currentStore, props);
        listeners.set(updateSelector, props);
      }, [props]);
      useEffect(function () {
        isMountedRef.current = true;
        return function () {
          listeners["delete"](updateSelector);
        };
      }, emptyArray);
      return stateRef.current;
    };
  };

  for (var _i3 = 0, _Object$entries3 = Object.entries((options === null || options === void 0 ? void 0 : options.hooks) || {}); _i3 < _Object$entries3.length; _i3++) {
    _loop3();
  }

  return _extends({
    useStore: useStore,
    actions: _extends({}, actionsObj, {}, asyncActionsObj),
    hooks: hooksObj
  }, produceObj);
}

/**
 * Create React Context version of a **react-state-selector** global store
 *
 * @export
 * @template TStore
 * Any object, including array or complex objects like Map, Set or Classes. Even primitives would work.
 * @template THooks
 * Object that defines the custom hooks of the store.
 * @template TActions
 * Object that defines the custom actions of the store.
 * @template TAsyncActions
 * Object that defines the custom async actions of the store.
 * @param {Immutable<TStore>} initialStore
 * Initial state of the store
 * @param {{
 *     hooks?: THooks;
 *     actions?: TActions;
 *     devName?: string;
 *     devToolsInProduction?: boolean;
 *   }} [options]
 * Options object that may contain:
 * hooks: custom hooks;
 * actions: custom actions;
 * devName: name used to activate Redux DevTools;
 * devToolsInProduction: flag to manually activate the usage of the Redux DevTools in production
 * @returns {{
 *   Provider: FunctionComponent<{ debugName?: string }>;
 *   useStore: IUseStore<TStore>;
 *   useProduce: IUseProduce<TStore>;
 *   useActions: () => IActionsObj<TStore, typeof options> & IAsyncActionsObj<TStore, typeof options>;
 *   hooks: IHooksObj<TStore, typeof options>;
 * }}
 * Resulting object containing of the store.
 */

function createStoreContext(initialStore, options) {
  if (process.env.NODE_ENV !== "production") {
    for (var name in options === null || options === void 0 ? void 0 : options.hooks) {
      if (name.length < 4 || name.slice(0, 3) !== "use" || name.charAt(3) === name.charAt(3).toLowerCase()) {
        throw new Error("All hooks should follow the rules of hooks for naming and \"" + name + "\" doesn't");
      }
    }
  }

  if (process.env.NODE_ENV !== "production") {
    if ((options === null || options === void 0 ? void 0 : options.actions) && options.asyncActions) {
      var asyncActionsKeys = Object.keys(options.asyncActions);

      for (var key in options.actions) {
        if (asyncActionsKeys.includes(key)) {
          throw new Error("All the actions and asyncActions should have different names and \"" + key + "\" exists in both objects!");
        }
      }
    }
  }

  var createStoragePersistenceInstance = function createStoragePersistenceInstance(produceStore, debugName) {
    var _options$storagePersi;

    if (options === null || options === void 0 ? void 0 : (_options$storagePersi = options.storagePersistence) === null || _options$storagePersi === void 0 ? void 0 : _options$storagePersi.isActive) {
      if (typeof initialStore !== "object" || Array.isArray(initialStore)) throw new Error("For local storage persistence your store has to be an object");
      var persistenceKey;

      if (options.storagePersistence.persistenceKey) {
        persistenceKey = options.storagePersistence.persistenceKey;
      } else if (options.devName) {
        persistenceKey = options.devName;
      }

      if (!persistenceKey) throw new Error("You have to specify persistence key or devName");

      if (debugName) {
        persistenceKey += "-" + debugName;
      } else if (debugName === null) {
        persistenceKey += "-noProvider";
      }

      return connectPersistenceStorage({
        persistenceKey: persistenceKey,
        produce: produceStore,
        debounceWait: options.storagePersistence.debounceWait,
        persistenceMethod: options.storagePersistence.persistenceMethod,
        isSSR: options.storagePersistence.isSSR
      });
    }

    return null;
  };

  var createDevToolsInstance = function createDevToolsInstance(debugName) {
    if ((options === null || options === void 0 ? void 0 : options.devName) && (options.devToolsInProduction || process.env.NODE_ENV !== "production")) {
      var devToolsName;

      if (debugName) {
        devToolsName = options.devName + "-" + debugName;
      } else if (debugName === null) {
        devToolsName = options.devName + "-noProvider";
      } else {
        devToolsName = options.devName;
      }

      var devTools = connectDevTools(devToolsName);

      if (devTools) {
        devTools.init(initialStore);
      }

      return devTools;
    }

    return undefined;
  };

  var createProduceObj = function createProduceObj(storeRef) {
    return {
      produce: function produce$1(draft) {
        var _storeRef$storagePers;

        if (typeof draft !== "function") return storeRef.store;

        if (storeRef.devTools) {
          var produceFn = produceWithPatches(draft);
          var produceResult = produceFn(storeRef.store);
          storeRef.store = produceResult[0];
          storeRef.devTools.send({
            type: "produce",
            payload: produceResult[1]
          }, storeRef.store);
        } else {
          var _produceFn = produce(draft);

          storeRef.store = _produceFn(storeRef.store);
        }

        storeRef.listeners.forEach(function (props, listener) {
          listener(storeRef.store, props);
        });
        (_storeRef$storagePers = storeRef.storagePersistence) === null || _storeRef$storagePers === void 0 ? void 0 : _storeRef$storagePers.setState(storeRef.store);
        return storeRef.store;
      },
      asyncProduce: function (draft) {
        try {
          if (typeof draft !== "function") return Promise.resolve(storeRef.store);
          var storeDraft = createDraft(storeRef.store);
          return Promise.resolve(Promise.resolve(draft(storeDraft))).then(function () {
            var _storeRef$storagePers2;

            finishDraft(storeDraft, function (changes) {
              if (changes.length) {
                storeRef.store = applyPatches(storeRef.store, changes);

                if (storeRef.devTools) {
                  storeRef.devTools.send({
                    type: "asyncProduce",
                    payload: changes
                  }, storeRef.store);
                }

                storeRef.listeners.forEach(function (props, listener) {
                  listener(storeRef.store, props);
                });
              }
            });
            (_storeRef$storagePers2 = storeRef.storagePersistence) === null || _storeRef$storagePers2 === void 0 ? void 0 : _storeRef$storagePers2.setState(storeRef.store);
            return storeRef.store;
          });
        } catch (e) {
          return Promise.reject(e);
        }
      }
    };
  };

  var providerRefCallback = function providerRefCallback(debugName) {
    return function () {
      var storeRef = {
        store: initialStore,
        listeners: new Map(),
        devTools: createDevToolsInstance(debugName)
      };
      var produceObj = createProduceObj(storeRef);
      storeRef.produce = produceObj;
      storeRef.storagePersistence = createStoragePersistenceInstance(produceObj.produce, debugName);
      return Object.assign(storeRef, {
        produce: produceObj
      });
    };
  };

  var StoreContext = createContext({
    current: providerRefCallback(null)()
  });
  var Provider = memo(function (_ref) {
    var children = _ref.children,
        debugName = _ref.debugName;
    var initialRef = useMemo(providerRefCallback(debugName), emptyArray);
    var value = useRef(initialRef);
    return createElement(StoreContext.Provider, {
      value: value,
      children: children
    });
  });

  var useStore = function useStore() {
    var storeCtx = useContext(StoreContext);
    var update = useUpdate(storeCtx.current.storagePersistence);
    useIsomorphicLayoutEffect(function () {
      var globalListener = createSelector(function (s) {
        return s;
      }, function () {
        update();
      });
      storeCtx.current.listeners.set(globalListener, null);
      return function () {
        storeCtx.current.listeners["delete"](globalListener);
      };
    }, emptyArray);
    return storeCtx.current.store;
  };

  var useProduce = function useProduce() {
    var storeCtx = useContext(StoreContext);
    return storeCtx.current.produce;
  };

  var useActions = function useActions() {
    var storeCtx = useContext(StoreContext);
    var produceObj = useProduce();
    return useMemo(function () {
      if (storeCtx.current.actions === undefined) {
        var actionsObj = {};
        var asyncActionsObj = {};

        var _loop = function _loop() {
          var _Object$entries$_i = _Object$entries[_i],
              actionName = _Object$entries$_i[0],
              actionFn = _Object$entries$_i[1];

          actionsObj[actionName] = function () {
            for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
              args[_key] = arguments[_key];
            }

            var actionDraft = actionFn.apply(void 0, args.concat([produceObj.produce]));

            if (storeCtx.current.devTools) {
              var produceFn = produceWithPatches(actionDraft);
              var produceResult = produceFn(storeCtx.current.store);
              storeCtx.current.store = produceResult[0];
              storeCtx.current.devTools.send({
                type: actionName,
                payload: produceResult[1]
              }, storeCtx.current.store);
            } else {
              var _produceFn2 = produce(actionDraft);

              storeCtx.current.store = _produceFn2(storeCtx.current.store);
            }

            storeCtx.current.listeners.forEach(function (props, listener) {
              listener(storeCtx.current.store, props);
            });
            return storeCtx.current.store;
          };
        };

        for (var _i = 0, _Object$entries = Object.entries((options === null || options === void 0 ? void 0 : options.actions) || {}); _i < _Object$entries.length; _i++) {
          _loop();
        }

        var _loop2 = function _loop2() {
          var _Object$entries2$_i = _Object$entries2[_i2],
              actionName = _Object$entries2$_i[0],
              actionFn = _Object$entries2$_i[1];

          asyncActionsObj[actionName] = function () {
            try {
              return Promise.resolve(actionFn(produceObj.produce).apply(void 0, arguments)).then(function () {
                return storeCtx.current.store;
              });
            } catch (e) {
              return Promise.reject(e);
            }
          };
        };

        for (var _i2 = 0, _Object$entries2 = Object.entries((options === null || options === void 0 ? void 0 : options.asyncActions) || {}); _i2 < _Object$entries2.length; _i2++) {
          _loop2();
        }

        storeCtx.current.actions = _extends({}, actionsObj, {}, asyncActionsObj);
      }

      return storeCtx.current.actions;
    }, emptyArray);
  };

  var hooksObj = {};

  var _loop3 = function _loop3() {
    var _Object$entries3$_i = _Object$entries3[_i3],
        hookName = _Object$entries3$_i[0],
        hookSelector = _Object$entries3$_i[1];

    hooksObj[hookName] = function (hooksProps, hookPropsDeps) {
      var storeCtx = useContext(StoreContext);
      var update = useUpdate(storeCtx.current.storagePersistence);
      var props = useMemo(toAnonFunction(hooksProps), hookPropsDeps || [hooksProps]);
      var isMountedRef = useRef(false);

      var _useMemo = useMemo(function () {
        return {
          updateSelector: createSelector(hookSelector, function (result) {
            stateRef.current = result;

            if (!isMountedRef.current) {
              return;
            }

            update();
          }),
          initialStateRef: hookSelector(storeCtx.current.store, props)
        };
      }, emptyArray),
          updateSelector = _useMemo.updateSelector,
          initialStateRef = _useMemo.initialStateRef;

      var stateRef = useRef(initialStateRef);
      useIsomorphicLayoutEffect(function () {
        updateSelector(storeCtx.current.store, props);
        storeCtx.current.listeners.set(updateSelector, props);
      }, [props]);
      useEffect(function () {
        isMountedRef.current = true;
        return function () {
          storeCtx.current.listeners["delete"](updateSelector);
        };
      }, emptyArray);
      return stateRef.current;
    };
  };

  for (var _i3 = 0, _Object$entries3 = Object.entries((options === null || options === void 0 ? void 0 : options.hooks) || {}); _i3 < _Object$entries3.length; _i3++) {
    _loop3();
  }

  return {
    Provider: Provider,
    useStore: useStore,
    useProduce: useProduce,
    useActions: useActions,
    hooks: hooksObj
  };
}

export { createStore, createStoreContext };
//# sourceMappingURL=react-state-selector.esm.js.map
